@@ -1,6 +1,6 @@
 ;;; polymode-test-utils.el --- Testing utilities for polymode -*- lexical-binding: t -*-
 ;;
-;; Copyright (C) 2018-2019, Vitalie Spinu
+;; Copyright (C) 2018-2022  Free Software Foundation, Inc.
 ;; Author: Vitalie Spinu
 ;; URL: https://github.com/polymode/polymode
 ;;
@@ -32,6 +32,7 @@
 
 (require 'ert)
 (require 'polymode)
+(require 'poly-lock)
 (eval-when-compile
   (require 'cl-lib))
 
@@ -40,9 +41,7 @@
 ;; (add-hook 'after-change-major-mode-hook #'global-font-lock-mode-enable-in-buffers)
 ;; (message "ACMH: %s  GFL:%s" after-change-major-mode-hook global-font-lock-mode)
 
-(setq ert-batch-backtrace-right-margin 200)
-(defvar pm-verbose (getenv "PM_VERBOSE"))
-
+(defvar pm-verbose nil)
 (defvar pm-test-current-change-set nil)
 (defun pm-test-get-file (name)
   "Find the file with NAME from inside a poly-xyz repo.
@@ -55,6 +54,10 @@ Look into tests/input directory then in samples directory."
                  if (file-exists-p f) return f)
         (error "No file with name '%s' found in '%s'" name default-directory))))
 
+(defun pm-test-running-on-github-p ()
+  (string= (downcase (or (getenv "GITHUB_ACTIONS") "false")) "true"))
+
+
 (defun pm-test-matcher (string span-alist matcher &optional dry-run)
   (with-temp-buffer
     (insert string)
@@ -83,8 +86,8 @@ Look into tests/input directory then in samples directory."
         (message ")"))
       nil)))
 
-(defmacro pm-test-run-on-string (mode string &rest body)
-  "Run BODY in a temporary buffer containing STRING in MODE.
+(defmacro pm-test-run-on-string (mode string position &rest body)
+  "Run BODY in a temporary buffer containing STRING in MODE at POSITION.
 MODE is a quoted symbol."
   (declare (indent 1) (debug (form form body)))
   `(let ((*buf* "*pm-test-string-buffer*"))
@@ -92,13 +95,14 @@ MODE is a quoted symbol."
        (kill-buffer *buf*))
      (with-current-buffer (get-buffer-create *buf*)
        (insert (substring-no-properties ,string))
-       (funcall ,mode)
+       (let ((inhibit-message (not pm-verbose)))
+         (funcall ,mode))
        (setq-default indent-tabs-mode nil)
        ;; In emacs 27 this is called from run-mode-hooks
        (and (bound-and-true-p syntax-propertize-function)
             (not (local-variable-p 'parse-sexp-lookup-properties))
             (setq-local parse-sexp-lookup-properties t))
-       (goto-char (point-min))
+       (funcall-interactively 'goto-char ,position)
        (let ((poly-lock-allow-background-adjustment nil))
          (when polymode-mode
            ;; font-lock not activated in batch mode
@@ -111,7 +115,7 @@ MODE is a quoted symbol."
 (defun pm-test-spans (mode string)
   (declare (indent 1))
   (pm-test-run-on-string mode
-    string
+    string 1
     (pm-map-over-spans
      (lambda (span)
        (let ((range0 (pm-span-to-range span)))
@@ -152,12 +156,11 @@ MODE is a quoted symbol."
          (remove-hook 'text-mode-hook 'flyspell-mode) ;; triggers "too much reentrancy" error
          (let ((inhibit-message (not pm-verbose)))
            (funcall-interactively ',mode))
-         ;; (flyspell-mode -1) ;; triggers "too much reentrancy" error
          (hack-local-variables 'ignore-mode)
          (goto-char (point-min))
          ,pre-form
+         ;; FIXME: figure this mambo-jumbo
          ;; need this to activate all chunks
-         (font-lock-ensure)
          (goto-char (point-min))
          (save-excursion
            (let ((font-lock-mode t))
@@ -175,7 +178,8 @@ MODE is a quoted symbol."
                 ;; fontification in X either (waf?)
                 (add-hook 'after-change-functions #'pm-test-invoke-fontification t t))
               (point-min) (point-max))))
-         (font-lock-ensure)
+         ;; (font-lock-flush)
+         ;; (font-lock-ensure)
          ,@body
          (current-buffer)))))
 
@@ -190,7 +194,7 @@ MODE is a quoted symbol."
            (smode major-mode)
            (stext (buffer-substring-no-properties sbeg send))
            ;; other buffer
-           (ref-buf (pm-test-run-on-string smode stext))
+           (ref-buf (pm-test-run-on-string smode stext 1))
            (ref-pos 1))
       (when pm-verbose
         (message "---- testing %s ----" (pm-format-span span t)))
@@ -243,10 +247,10 @@ MODE is a quoted symbol."
                      :pos pos
                      :ref-pos ref-pos
                      :line (progn (goto-char pos)
-                                  (buffer-substring-no-properties (point-at-bol) (point-at-eol)))
+                                  (buffer-substring-no-properties (line-beginning-position) (line-end-position)))
                      :ref-line (with-current-buffer ref-buf
                                  (goto-char ref-pos)
-                                 (buffer-substring-no-properties (point-at-bol) (point-at-eol)))
+                                 (buffer-substring-no-properties (line-beginning-position) (line-end-position)))
                      :mode smode))))
               ;; for the interactive convenience
               (switch-to-buffer (current-buffer))
@@ -258,6 +262,8 @@ MODE is a quoted symbol."
 ALLOW-FAILED-FACES should be a list of faces on which failures
 are OK."
   (save-excursion
+    (font-lock-flush)
+    (font-lock-ensure)
     (pm-map-over-spans
      (lambda (span) (pm-test-span-faces span allow-failed-faces)))))
 
@@ -346,6 +352,7 @@ execution undo is called once. After each change-set
   `(kill-buffer
     (pm-test-run-on-file ,mode ,file
       (pm-test-faces)
+      (set-buffer-modified-p nil)
       (dolist (cset ',change-sets)
         (let ((poly-lock-defer-after-change nil)
               (pm-test-current-change-set (caar cset)))
@@ -356,7 +363,8 @@ execution undo is called once. After each change-set
           (undo-boundary)
           (pm-test-faces)
           (let ((inhibit-message (not pm-verbose)))
-            (undo)))))))
+            (when (buffer-modified-p)
+              (undo))))))))
 
 (defun pm-test--run-indentation-tests ()
   "Run an automatic batch of indentation tests.
@@ -367,7 +375,7 @@ points."
   (goto-char (point-min))
   (set-buffer-modified-p nil)
   (while (not (eobp))
-    (let ((orig-line (buffer-substring-no-properties (point-at-eol) (point-at-bol))))
+    (let ((orig-line (buffer-substring-no-properties (line-end-position) (line-beginning-position))))
       (unless (string-match-p "no-indent-test" orig-line)
         (undo-boundary)
         ;; (pm-switch-to-buffer)
@@ -375,12 +383,12 @@ points."
         ;;          (line-number-at-pos) (point) (current-buffer)
         ;;          (syntax-ppss) syntax-propertize--done)
         (pm-indent-line-dispatcher)
-        (unless (equal orig-line (buffer-substring-no-properties (point-at-eol) (point-at-bol)))
+        (unless (equal orig-line (buffer-substring-no-properties (line-end-position) (line-beginning-position)))
           (undo-boundary)
           (pm-switch-to-buffer (point))
           (ert-fail (list :pos (point) :line (line-number-at-pos)
                           :mode major-mode
-                          :indent-line (buffer-substring-no-properties (point-at-bol) (point-at-eol)))))))
+                          :indent-line (buffer-substring-no-properties (line-beginning-position) (line-end-position)))))))
     (forward-line 1))
   (let (points1 points2)
     (pm-map-over-spans (lambda (span) (push (/ (+ (nth 1 span) (nth 2 span)) 2) points1)))
@@ -432,15 +440,16 @@ points."
                              :ref (with-temp-buffer
                                     (insert right)
                                     (goto-char pos)
-                                    (buffer-substring-no-properties (point-at-bol) (point-at-eol)))
+                                    (buffer-substring-no-properties (line-beginning-position) (line-end-position)))
                              :new (progn
                                     (goto-char pos)
-                                    (buffer-substring-no-properties (point-at-bol) (point-at-eol)))))))))))
+                                    (buffer-substring-no-properties (line-beginning-position) (line-end-position)))))))))))
 
 (defmacro pm-test-map-over-modes (mode file)
   `(pm-test-run-on-file ,mode ,file
      (let ((beg (point-min))
-           (end (point-max)))
+           (end (point-max))
+           (pm-use-cache t))
        (with-buffer-prepared-for-poly-lock
         (remove-text-properties beg end '(:pm-span :pm-face)))
        (pm-map-over-modes (lambda (b e)) beg end)
