@@ -1,10 +1,10 @@
 ;;; polymode.el --- Extensible framework for multiple major modes -*- lexical-binding: t -*-
 ;;
 ;; Author: Vitalie Spinu
-;; Maintainer: Vitalie Spinu
-;; Copyright (C) 2013-2019, Vitalie Spinu
-;; Package-Version: 0.2.2
-;; Package-Revision: 44265e35161d
+;; Maintainer: Vitalie Spinu <spinuvit@gmail.com>
+;; Copyright (C) 2013-2022  Free Software Foundation, Inc.
+;; Package-Version: 20251217.1327
+;; Package-Revision: a48648fe2b2e
 ;; Package-Requires: ((emacs "25"))
 ;; URL: https://github.com/polymode/polymode
 ;; Keywords: languages, multi-modes, processes
@@ -47,40 +47,49 @@
 (require 'easymenu)
 (require 'derived)
 
-(defvar polymode-prefix-key nil
-  "[Obsoleted] Prefix key for the polymode mode keymap.
-Not effective after loading the polymode library.")
-(make-obsolete-variable 'polymode-prefix-key "Unbind in `polymode-mode-map'" "v0.1.6")
+(defvar polymode-prefix-key "\M-n"
+  "Default prefix key in `polymode-minor-mode-map'.
+Not effective after loading the polymode library.
+
+Instead of setting this key you can programatically bind it directly
+in `polymode-minor-mode-map` keymap:
+
+ (define-key polymode-minor-mode-map (kbd \"M-n\") nil)
+ ;unbind the default M-n prefix
+ (define-key polymode-minor-mode-map (kbd \"C-c n\") polymode-map)
+")
 
 (defvar polymode-map
   (let ((map (define-prefix-command 'polymode-map)))
     ;; eval
     (define-key map "v" 'polymode-eval-map)
     ;; navigation
-    (define-key map "\C-n" 'polymode-next-chunk)
-    (define-key map "\C-p" 'polymode-previous-chunk)
-    (define-key map "\C-\M-n" 'polymode-next-chunk-same-type)
-    (define-key map "\C-\M-p" 'polymode-previous-chunk-same-type)
+    (define-key map "\C-n" #'polymode-next-chunk)
+    (define-key map "\C-p" #'polymode-previous-chunk)
+    (define-key map "\C-\M-n" #'polymode-next-chunk-same-type)
+    (define-key map "\C-\M-p" #'polymode-previous-chunk-same-type)
     ;; chunk manipulation
-    (define-key map "\M-k" 'polymode-kill-chunk)
-    (define-key map "\M-m" 'polymode-mark-or-extend-chunk)
-    (define-key map "\C-t" 'polymode-toggle-chunk-narrowing)
+    (define-key map "\M-k" #'polymode-kill-chunk)
+    (define-key map "\M-m" #'polymode-mark-or-extend-chunk)
+    (define-key map "\M-w" #'polymode-kill-ring-save-chunk)
+    (define-key map "\C-t" #'polymode-toggle-chunk-narrowing)
     ;; backends
-    (define-key map "e" 'polymode-export)
-    (define-key map "E" 'polymode-set-exporter)
-    (define-key map "w" 'polymode-weave)
-    (define-key map "W" 'polymode-set-weaver)
-    (define-key map "t" 'polymode-tangle)
-    (define-key map "T" 'polymode-set-tangler)
-    (define-key map "$" 'polymode-show-process-buffer)
+    (define-key map "e" #'polymode-export)
+    (define-key map "E" #'polymode-set-exporter)
+    (define-key map "w" #'polymode-weave)
+    (define-key map "W" #'polymode-set-weaver)
+    ;; (define-key map "t" #'polymode-tangle)
+    ;; (define-key map "T" #'polymode-set-tangler)
+    (define-key map "$" #'polymode-show-process-buffer)
     map)
   "Polymode prefix map.
-Lives on `polymode-prefix-key' in polymode buffers.")
+By default, lives on `polymode-prefix-key' in polymode buffers.")
 
 (defvaralias 'polymode-mode-map 'polymode-minor-mode-map)
 (defvar polymode-minor-mode-map
   (let ((map (make-sparse-keymap)))
-    (define-key map (or polymode-prefix-key "\M-n") 'polymode-map)
+    (when polymode-prefix-key
+      (define-key map polymode-prefix-key 'polymode-map))
     map)
   "The minor mode keymap which is inherited by all polymodes.")
 
@@ -237,6 +246,23 @@ Return the number of chunks of the same type moved over."
                 (pm-span-to-range span)
               (pm-chunk-range (1- (nth 1 span))))))))
 
+(defun polymode-kill-ring-save-chunk ()
+  "Copy current chunk into the kill-ring.
+When in the head of chunk, copy the chunk including the head and tail,
+otherwise only the body span.
+When called interactively, highlight the copie region for `copy-region-blink-delay'."
+  (interactive)
+  (let ((span (pm-innermost-span)))
+    (let ((range (if (memq (car span) '(nil body))
+                     (pm-span-to-range span)
+                   (pm-chunk-range))))
+      (copy-region-as-kill (car range) (cdr range))
+      (when (called-interactively-p 'interactive)
+        (let ((overlay (make-overlay (car range) (cdr range))))
+          (overlay-put overlay 'face  'highlight)
+          (run-with-timer copy-region-blink-delay nil
+                          (lambda () (delete-overlay overlay))))))))
+
 (defun polymode-mark-or-extend-chunk ()
   "DWIM command to repeatedly mark chunk or extend region.
 When no region is active, mark the current span if in body of a
@@ -436,7 +462,7 @@ non-nil, don't throw if `polymode-eval-region-function' is nil."
         (pi parent-conf)
         (parent-map))
     (while pi
-      (let ((map (and (slot-boundp pi :keylist)
+      (let ((map (and (slot-boundp pi 'keylist)
                       (eieio-oref pi 'keylist))))
         (when map
           (if (and (symbolp map)
@@ -446,7 +472,7 @@ non-nil, don't throw if `polymode-eval-region-function' is nil."
               (setq parent-map map
                     pi nil)
             ;; list, descend to next parent and append the key list to keylist
-            (setq pi (and (slot-boundp pi :parent-instance)
+            (setq pi (and (slot-boundp pi 'parent-instance)
                           (eieio-oref pi 'parent-instance))
                   keylist (append map keylist))))))
     (when (and parent-map (symbolp parent-map))
@@ -521,6 +547,7 @@ most frequently used slots are:
 :innermodes List of symbols pointing to `pm-inner-chunkmode'
   objects which specify the behavior of inner modes (or submodes)."
   (declare
+   (indent defun)
    (doc-string 3)
    (debug (&define name
                    [&optional [&not keywordp] name]
@@ -552,10 +579,10 @@ most frequently used slots are:
     (while (keywordp (setq keyw (car body)))
       (setq body (cdr body))
       (pcase keyw
-        (`:lighter (setq lighter (purecopy (pop body))))
-        (`:keymap (setq keymap (pop body)))
-        (`:after-hook (setq after-hook (pop body)))
-        (`:keylist (setq keylist (pop body)))
+        (:lighter (setq lighter (purecopy (pop body))))
+        (:keymap (setq keymap (pop body)))
+        (:after-hook (setq after-hook (pop body)))
+        (:keylist (setq keylist (pop body)))
         (_ (push (pop body) slots) (push keyw slots))))
 
 
@@ -625,9 +652,7 @@ most frequently used slots are:
              (setq ,mode state)
              (if state
                  (unless (buffer-base-buffer)
-                   ;; Call in indirect buffers only. Inner modes during
-                   ;; initialization call this polymode minor-mode which triggers
-                   ;; this `pm-initialize'.
+                   ;; Call in host (base) buffers only.
                    (when ,mode
                      (let ((obj (clone ,config-name)))
                        ;; (eieio-oset obj '-minor-mode ',mode)
@@ -660,7 +685,7 @@ most frequently used slots are:
 
 (define-minor-mode polymode-minor-mode
   "Polymode minor mode, used to make everything work."
-  nil " PM")
+  :lighter " PM")
 
 (define-derived-mode poly-head-tail-mode prog-mode "HeadTail"
   "Default major mode for polymode head and tail spans."
